@@ -609,6 +609,8 @@ class ResNetFedAvgScheduler(AutoFedAvgScheduler):
 
     def __init__(self,
                  auto_proxy: AutoResNetFedAvg,
+                 min_clients: int,
+                 max_clients: int,
                  max_rounds: int = 0,
                  merge_epochs: int = 1,
                  calculation_timeout: int = 300,
@@ -616,6 +618,8 @@ class ResNetFedAvgScheduler(AutoFedAvgScheduler):
                  log_rounds: int = 0,
                  involve_aggregator: bool = False):
         super().__init__(auto_proxy=auto_proxy,
+                         min_clients=min_clients,
+                         max_clients=max_clients,
                          max_rounds=max_rounds,
                          merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
@@ -642,7 +646,7 @@ class ResNetFedAvgScheduler(AutoFedAvgScheduler):
     def train_an_epoch(self):
         self.auto_proxy.train_an_epoch()
 
-    def run_test(self):
+    def test(self):
         avg_loss, correct_rate = self.auto_proxy.run_test()
 
         self.tb_writer.add_scalar('test_results/avg_loss', avg_loss, self.current_round)
