@@ -654,7 +654,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                                                   data_stream=sync_data.to_bytes())
                     self.push_log('Successfully synchronized context with the new aggregatro.')
 
-                if len(self._calculators) == len(self._participants):  # all finished
+                if len(self._calculators) == len(self._participants) - 1:  # all finished
                     break
 
             elif isinstance(_event, CheckinEvent):
@@ -758,7 +758,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         self.contractor.notify_ready_for_aggregation(round=self.current_round)
         self.push_log('Now waiting for executing calculation ...')
         accum_result, result_count = self._wait_for_calculation()
-        if result_count < len(self.gethered) + int(self.involve_aggregator):
+        if result_count < len(self._calculators):
             self.push_log('Task failed because of too few calculation results gathered.')
             raise AggregationError(f'Too few results gathered: {result_count} copies.')
         self.push_log(f'Received {result_count} copies of calculation results.')
@@ -819,7 +819,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                     accum_result[_key].add_(_new_state_dict[_key])
                 result_count += 1
                 self.push_log(f'Received calculation results from ID: {_event.source}')
-                if result_count == len(self.gethered) + int(self.involve_aggregator):
+                if result_count == len(self._calculators):
                     return accum_result, result_count
 
             elif isinstance(_event, CheckinEvent):
