@@ -4,7 +4,7 @@ import random
 import shutil
 from abc import abstractmethod
 from tempfile import TemporaryFile
-from typing import List, Tuple
+from typing import List, Set, Tuple
 
 import tenseal as ts
 import torch
@@ -112,7 +112,7 @@ class SecureHeteroNNCollaboratorScheduler(SecureHeteroNNScheduler):
     @abstractmethod
     def iterate_train_feature(self,
                               feature_model: nn.Module,
-                              train_ids: List[str]) -> torch.Tensor:
+                              train_ids: Set[str]) -> torch.Tensor:
         """Iterate over train dataset and features a batch of data each time.
 
         :args
@@ -125,7 +125,7 @@ class SecureHeteroNNCollaboratorScheduler(SecureHeteroNNScheduler):
     @abstractmethod
     def iterate_test_feature(self,
                              feature_model: nn.Module,
-                             test_ids: List[str]) -> torch.Tensor:
+                             test_ids: Set[str]) -> torch.Tensor:
         """Iterate over test dataset and features a batch of data each time.
 
         :args
