@@ -90,8 +90,8 @@ class mock_context:
         mock_context._mock_context__node_id = self.id
 
         if self.clean:
-            _mock_cur.execute('DELETE * FROM contract_consume_cursors')
-            _mock_cur.execute('DELETE * FROM contracts')
+            _mock_cur.execute('DELETE FROM contract_consume_cursors WHERE 1=1')
+            _mock_cur.execute('DELETE FROM contracts WHERE 1=1')
             _mock_cur.connection.commit()
         self._init_contract_cursor(node_id=self.id)
 
@@ -101,15 +101,12 @@ class mock_context:
             (node_id,)
         ).fetchone()
         if is_exist is None:
-            max_id = _mock_cur.execute(
-                'SELECT MAX(contract_id) FROM contracts'
-            ).fetchone()['MAX(contract_id)'] or 0
             _mock_cur.execute(
                 '''
                 INSERT INTO contract_consume_cursors (node_id, cursor)
                 VALUES (?, ?)
                 ''',
-                (node_id, max_id)
+                (node_id, 0)
             )
             _mock_cur.connection.commit()
 
